! =================================================================================================================================
! MODULE 	: stomate_vmax
!
! CONTACT	: orchidee-help _at_ ipsl.jussieu.fr
!
! LICENCE      	: IPSL (2006). This software is governed by the CeCILL licence see ORCHIDEE/ORCHIDEE_CeCILL.LIC
!
!>\BRIEF        calculates the leaf efficiency.
!!	
!!\n DESCRIPTION: None
!!
!! RECENT CHANGE(S): None
!!
!! SVN		:
!! $HeadURL$ 
!! $Date$
!! $Revision$
!! \n
!_ =================================================================================================================================

MODULE stomate_vmax

  ! modules used:

  USE ioipsl_para
  USE stomate_data
  USE constantes
  USE pft_parameters

  IMPLICIT NONE

  ! private & public routines

  PRIVATE
  PUBLIC vmax_prognostic, vmax_diagnostic, vmax_clear

  ! first call
  LOGICAL, SAVE                                              :: firstcall = .TRUE.
!$OMP THREADPRIVATE(firstcall)

CONTAINS

!! ================================================================================================================================
!! SUBROUTINE	: vmax_clear
!!
!>\BRIEF	  Flag setting 
!!
!!\n DESCRIPTION: This subroutine sets flags ::firstcall, to .TRUE., and therefore activates   
!!		  section 1.1 of the ::vmax subroutine which writes messages to the output. \n
!!		  This subroutine is called at the end of the subroutine ::stomate_clear, in the 
!!		  module ::stomate.
!!
!! RECENT CHANGE(S):None
!!
!! MAIN OUTPUT VARIABLE(S): ::firstcall
!!
!! REFERENCE(S)  : None 
!!
!! FLOWCHART     : None
!! \n		  
!_ =================================================================================================================================

  SUBROUTINE vmax_clear
    firstcall=.TRUE.
  END SUBROUTINE vmax_clear



!! ================================================================================================================================
!! SUBROUTINE    : vmax
!!
!>\BRIEF         This subroutine computes vcmax photosynthesis parameters 
!! given optimal vcmax parameter value and a leaf age-related efficiency.
!!
!! DESCRIPTION (functional, design, flags): 
!! Leaf age classes are introduced to take into account the fact that photosynthetic activity depends on leaf age
!! (Ishida et al., 1999). There are \f$nleafages\f$ classes (constant defined in pft_parameters.f90).
!! This subroutine first calculates the new age of each leaf age-class based on fraction of leaf 
!! that goes from one to another class.                                              
!! Then calculation of the new fraction of leaf in each class is performed.      
!! Last, leaf efficiency is calculated for each PFT and for each leaf age class.
!! vcmax is defined as vcmax25 weighted by a mean leaf
!! efficiency. vcmax25 is PFT-dependent constants defined in pft_parameters.f90.
!!
!! This routine is called once at the beginning by stomate_var_init and then at each stomate time step by stomateLpj.
!!
!! RECENT CHANGE(S): None
!!
!! MAIN OUTPUT VARIABLE(S): vcmax
!!
!! REFERENCE(S)	: 
!! - Ishida, A., A. Uemura, N. Koike, Y. Matsumoto, and A. Lai Hoe (1999),
!! Interactive effects of leaf age and self-shading on leaf structure, photosynthetic
!! capacity and chlorophyll fluorescence in the rain forest tree,
!! dryobalanops aromatica, Tree Physiol., 19, 741-747
!!
!! FLOWCHART    : None
!!
!! REVISION(S)	: None
!! \n
!_ ================================================================================================================================

  SUBROUTINE vmax_prognostic (npts, dt, leaf_age, leaf_frac, &
       age_stand, vcmax, tau_eff_leaf)

    !
    !! 0. Variable and parameter declaration
    !

    !
    !! 0.1 Input variables
    !
    INTEGER(i_std), INTENT(in)                                 :: npts                    !! Domain size (unitless)
    REAL(r_std), INTENT(in)                                    :: dt                      !! time step of stomate (days)
    INTEGER(i_std),DIMENSION(npts,nvm),INTENT(in)              :: age_stand               !! Age of stand (years)
                                                                                          !! (is the forest being managed?)
    REAL(r_std), DIMENSION(:,:), INTENT(in)                    :: tau_eff_leaf            !! Effective leaf turnover time that accounts
                                                                                          !! waterstress (days)
    !
    !! 0.2 Output variables 
    !
    REAL(r_std), DIMENSION(npts,nvm), INTENT(out)              :: vcmax                   !! Maximum rate of carboxylation 
                                                                                          !! @tex ($\mu mol m^{-2} s^{-1}$) @endtex

    !
    !! 0.3 Modified variables
    !
    REAL(r_std), DIMENSION(npts,nvm,nleafages), INTENT(inout)  :: leaf_age                !! Leaf age (days)
    REAL(r_std), DIMENSION(npts,nvm,nleafages), INTENT(inout)  :: leaf_frac               !! fraction of leaves in leaf age 
                                                                                          !! classes 
                                                                                          !! (unitless)

    !
    !! 0.4 Local variables
    !
    REAL(r_std), DIMENSION(npts)                               :: leaf_efficiency         !! leaf efficiency (vcmax/vcmax25)
                                                                                          !! (unitless)
    REAL(r_std), DIMENSION(npts,nvm,nleafages)                 :: d_leaf_frac             !! turnover between age classes
                                                                                          !! (unitless)
    REAL(r_std), DIMENSION(npts,nleafages)                     :: leaf_age_new            !! new leaf age (days)
    REAL(r_std), DIMENSION(npts)                               :: sumfrac                 !! sum of leaf age fractions, 
                                                                                          !! for normalization
                                                                                          !! (unitless)
    REAL(r_std), DIMENSION(npts)                               :: rel_age                 !! relative leaf age (age/critical age)
                                                                                          !! (unitless)
    INTEGER(i_std)                                             :: i,j,m                   !! indices (unitless)
    REAL(r_std), DIMENSION(npts,nvm)                           :: age_decline             !! Age-related decline in photosynthesis efficiency 
                                                                                          !! (as a proportion to vcmax and vjmax : 1 while age<60
                                                                                          !! (cf. Gower 1996) 
                                                                                          !!  and decline_max when age>140 (cf Gower 1996))
    REAL(r_std), SAVE                                          :: ss_decl_max             !! Temporary variable for sensitivity study 
!$OMP THREADPRIVATE(ss_decl_max)

!_ ================================================================================================================================

    IF (bavard.GE.3) WRITE(numout,*) 'Entering vmax'

    !
    !! 1 Initialization
    !

    !
    !! 1.1 first call: info about flags and parameters.
    !

    IF ( firstcall ) THEN

! We do not think we need the FM stuff here anymore.  In theory,
! the only difference between managed trees and unmanaged trees
! should be the thinning operations.
!!$       IF ( control%forest_management ) THEN
!!$          
!!$          ss_decl_max = 1.
!!$          !Config Key   = 
!!$          !Config Desc  = 
!!$          !Config If    = 
!!$          !Config Def   = 
!!$          !Config Help  = 
!!$          !Config Units = 
!!$          CALL getin_p ('ss_decl_max',ss_decl_max)
!!$         
!!$       END IF ! control%forest_management

       WRITE(numout,*) 'vmax:'

       WRITE(numout,*) '   > offset (minimum vcmax/vmax_opt):' , vmax_offset
       WRITE(numout,*) '   > relative leaf age at which vmax reaches vcmax_opt:', leafage_firstmax 
       WRITE(numout,*) '   > relative leaf age at which vmax falls below vcmax_opt:', leafage_lastmax
       WRITE(numout,*) '   > relative leaf age at which vmax reaches its minimum:', leafage_old

       firstcall = .FALSE.

    ENDIF

    !
    !! 1.2 initialize output
    !
    age_decline(:,:) = zero
    vcmax(:,:) = zero

    !
    !! 2 leaf age: general increase and turnover between age classes.
    !

    !
    !! 2.1 increase leaf age
    !



!
!! The age of the leaves in each leaf-age-class increases by 1 time step.
    DO m = 1, nleafages	! Loop over # leaf age classes 
       DO j = 2,nvm	! Loop over # PFTs
          WHERE ( leaf_frac(:,j,m) .GT. min_stomate )

             leaf_age(:,j,m) = leaf_age(:,j,m) + dt
             
          ENDWHERE
       ENDDO	! Loop over # PFTs 

    ENDDO	! Loop over # leaf age classes

   

    !
    !! 2.2 turnover between leaf age classes
    !     d_leaf_frac(:,:,m) = what leaves m-1 and goes into m
    !

    DO j = 2,nvm	! Loop over # PFTs

       !! 2.2.1 fluxes

       !! nothing goes into first age class
       d_leaf_frac(:,j,1) = zero

       !! for others age classes (what goes from m-1 to m)
       DO m = 2, nleafages 

          !! leaf_timecst is defined in pft_parameters.f90 as the quotient of the critical leaf age per the number of age classes.
          !! The critical leaf age is a PFT-dependent constant defined in pft_parameters.f90, that represents the leaf life span.
          !! This time constant (leaf_timecst) determines the turnover between the nleafages different leaf age classes
          !! (see section [118] in Krinner et al. (2005)).
          d_leaf_frac(:,j,m) = leaf_frac(:,j,m-1) * dt/leaf_timecst(j)
          !KIM 
          !!!! hiding this for doing global runs
          !WRITE(numout,*) 'turnover between leaf classes, pft, age class', d_leaf_frac(:,j,m), j, m

          !KIM 
     !     IF (j .EQ. 4) THEN
     !       WRITE(numout,*) 'turnover between leaf classes, pft, age class', d_leaf_frac(:,j,m), j, m
     !     ENDIF

       ENDDO

       !! 2.2.2 new leaf age in class
       !!       new age = ( old age * (old fraction - fractional loss) + fractional increase * age of the source class ) / new fraction
       !!       The leaf age of the youngest class (m=1) is updated into stomate_alloc          
       leaf_age_new(:,:) = zero

       DO m = 2, nleafages-1       ! Loop over age classes
	!! For all age classes except first and last 
          WHERE ( d_leaf_frac(:,j,m) .GT. min_stomate )

             leaf_age_new(:,m) = ( ( (leaf_frac(:,j,m)- d_leaf_frac(:,j,m+1)) * leaf_age(:,j,m) )  + &
                  ( d_leaf_frac(:,j,m) * leaf_age(:,j,m-1) ) ) / &
                  ( leaf_frac(:,j,m) + d_leaf_frac(:,j,m)- d_leaf_frac(:,j,m+1) )
             
          ENDWHERE

       ENDDO       ! Loop over age classes

	!! For last age class, there is no leaf fraction leaving the class. 

       WHERE ( d_leaf_frac(:,j,nleafages) .GT. min_stomate )

          leaf_age_new(:,nleafages) = ( ( leaf_frac(:,j,nleafages) * leaf_age(:,j,nleafages) )  + &
               ( d_leaf_frac(:,j,nleafages) * leaf_age(:,j,nleafages-1) ) ) / &
               ( leaf_frac(:,j,nleafages) + d_leaf_frac(:,j,nleafages) )


       ENDWHERE
       !KIM
       !!! hiding these write statements for global runs
       !WRITE(numout,*) 'new leaf age, age class', leaf_age_new(:,:)
       !WRITE(numout,*) 'old leaf age, pft, age class', leaf_age(:,j,:),j

                        !KIM
     !  IF (j .EQ. 4) THEN
     !     WRITE(numout,*) 'new leaf age, age class', leaf_age_new
     !     WRITE(numout,*) 'old leaf age, pft, age', leaf_age,j
     !  ENDIF

       DO m = 2, nleafages       ! Loop over age classes

          WHERE ( d_leaf_frac(:,j,m) .GT. min_stomate )

             leaf_age(:,j,m) = leaf_age_new(:,m)

          ENDWHERE

       ENDDO       ! Loop over age classes

       !! 2.2.3 calculate new fraction

       DO m = 2, nleafages       ! Loop over age classes

          ! where the change comes from
          leaf_frac(:,j,m-1) = leaf_frac(:,j,m-1) - d_leaf_frac(:,j,m)

          ! where it goes to
          leaf_frac(:,j,m) = leaf_frac(:,j,m) + d_leaf_frac(:,j,m)

       ENDDO       ! Loop over age classes

       !! 2.2.4 renormalize fractions in order to prevent accumulation 
       !       of numerical errors

       ! correct small negative values

       DO m = 1, nleafages
          leaf_frac(:,j,m) = MAX( zero, leaf_frac(:,j,m) )
       ENDDO

       ! total of fractions, should be very close to one where there is leaf mass

       sumfrac(:) = zero

       DO m = 1, nleafages       ! Loop over age classes

          sumfrac(:) = sumfrac(:) + leaf_frac(:,j,m)

       ENDDO       ! Loop over age classes

       ! normalize

       DO m = 1, nleafages       ! Loop over age classes



          WHERE ( sumfrac(:) .GT. min_stomate )

             leaf_frac(:,j,m) = leaf_frac(:,j,m) / sumfrac(:) 



          ELSEWHERE

             leaf_frac(:,j,m) = zero



          ENDWHERE



       ENDDO       ! Loop over age classes

    ENDDO         ! Loop over PFTs

    !
    !! 3 calculate vmax as a function of the age
    !

    DO j = 2,nvm

       vcmax(:,j) = zero


       ! sum up over the different age classes

       DO m = 1, nleafages       ! Loop over age classes

          !
          !! 3.1 efficiency in each of the age classes
          !!     it varies from vmax_offset to 1 
          !!     linearly increases from vmax_offset to 1 for 0 < rel_age < leafage_firstmax
          !!     is 1 when leafage_firstmax < rel_age < leafage_lastmax
          !!     linearly decreases from 1 to vmax_offset for leafage_lastmax < rel_age < leafage_firstmax
          !!     vmax_offset for rel_age >= leafage_old
          !!     (Ishida et al., 1999)

          rel_age(:) = leaf_age(:,j,m) / tau_eff_leaf(:,j)

          leaf_efficiency(:) = MAX( vmax_offset, MIN( un, &
               vmax_offset + (un - vmax_offset) * rel_age(:) / leafage_firstmax, &
               un - (un - vmax_offset) * ( rel_age(:) - leafage_lastmax ) / &
               ( leafage_old - leafage_lastmax ) ) )

          
          !KIM
           IF (ld_vmax .AND. j .EQ. test_pft .AND. m .EQ. 4) THEN 
              WRITE(numout,*) 'rel_age', rel_age(:)
              WRITE(numout,*) 'leaf efficiency', leaf_efficiency(:)
           ENDIF


          !
          !! 3.2 add to mean vmax
          !
          
          vcmax(:,j) = vcmax(:,j) + Vcmax25(j) * leaf_efficiency(:) * leaf_frac(:,j,m)



       ENDDO     ! loop over age classes
       !KIM
       IF (ld_vmax .AND. j .EQ. test_pft) THEN 
           WRITE(numout,*) 'vcmax corrected for leafage, pft', vcmax(:,test_pft), j
       ENDIF
       !
       !! 4 Modify vmax as a function of the age of the stand and management
       !
! We do not think we need the FM stuff here anymore.  In theory,
! the only difference between managed trees and unmanaged trees
! should be the thinning operations.
!!$       IF ( control%forest_management ) THEN
!!$
!!$          IF (is_tree(j)) THEN
!!$
!!$             WHERE ( forest_managed(:,j) == 0 )
!!$                age_decline(:,j) = 1.        !-ss_decl_max*decl_factor(j)*100.
!!$             END WHERE
!!$
!!$             DO i = 1,npts
!!$                IF (forest_managed(i,j) > 0) THEN
!!$
!!$                   ! If TCR, values from Calfapietra 2005
!!$                   IF (forest_managed(i,j) == 4) THEN
!!$                      vcmax(i,j) = vcmax(i,j)*79.1/vcmax_opt(j)
!!$                      vjmax(i,j) = vjmax(i,j)*152.4/vjmax_opt(j)
!!$                   END IF
!!$                   ! Final age-related decline in photosynthesis efficiency
!!$                   age_decline(i,j) = 1.-ss_decl_max*decl_factor(j)*REAL(MIN(MAX(age_stand(i,j)-50,0),100)) !**0.5
!!$
!!$                END IF
!!$             END DO
!!$
!!$             vcmax(:,j) = age_decline(:,j)**4*opt_factor(j)*vcmax(:,j)
!!$             vjmax(:,j) = age_decline(:,j)**4*opt_factor(j)*vjmax(:,j)
!!$
!!$          ENDIF
!!$
!!$       END IF ! (control%forest_management)

    ENDDO       ! loop over PFTs

    IF (bavard.GE.4) WRITE(numout,*) 'Leaving vmax'

  END SUBROUTINE vmax_prognostic

!! ================================================================================================================================
!! SUBROUTINE    : vmax
!!
!>\BRIEF         This subroutine computes vcmax photosynthesis parameters 
!! given optimal vcmax parameter value and a leaf age-related efficiency.
!!
!! DESCRIPTION (functional, design, flags): 
!! Leaf age classes are introduced to take into account the fact that photosynthetic activity depends on leaf age
!! (Ishida et al., 1999). There are \f$nleafages\f$ classes (constant defined in pft_parameters.f90).
!! This subroutine first calculates the new age of each leaf age-class based on fraction of leaf 
!! that goes from one to another class.                                              
!! Then calculation of the new fraction of leaf in each class is performed.      
!! Last, leaf efficiency is calculated for each PFT and for each leaf age class.
!! vcmax is defined as vcmax25 weighted by a mean leaf
!! efficiency. vcmax25 is PFT-dependent constants defined in pft_parameters.f90.
!!
!! This routine is called once at the beginning by stomate_var_init and then at each stomate time step by stomateLpj.
!!
!! RECENT CHANGE(S): None
!!
!! MAIN OUTPUT VARIABLE(S): vcmax
!!
!! REFERENCE(S)	: 
!! - Ishida, A., A. Uemura, N. Koike, Y. Matsumoto, and A. Lai Hoe (1999),
!! Interactive effects of leaf age and self-shading on leaf structure, photosynthetic
!! capacity and chlorophyll fluorescence in the rain forest tree,
!! dryobalanops aromatica, Tree Physiol., 19, 741-747
!!
!! FLOWCHART    : None
!!
!! REVISION(S)	: None
!! \n
!_ ================================================================================================================================

  SUBROUTINE vmax_diagnostic (npts, dt, &
       leaf_age, leaf_frac, &
       forest_managed, age_stand, &
       vcmax)

    !
    !! 0. Variable and parameter declaration
    !

    !
    !! 0.1 Input variables
    !
    INTEGER(i_std), INTENT(in)                                 :: npts                    !! Domain size (unitless)
    REAL(r_std), INTENT(in)                                    :: dt                      !! time step of stomate (days)
    INTEGER(i_std),DIMENSION(npts,nvm),INTENT(in)              :: age_stand               !! Age of stand (years)
    INTEGER(i_std), DIMENSION (npts,nvm), INTENT(in)           :: forest_managed          !! forest management flag 
                                                                                          !! (is the forest being managed?)
    !
    !! 0.2 Output variables 
    !
    REAL(r_std), DIMENSION(npts,nvm), INTENT(out)              :: vcmax                   !! Maximum rate of carboxylation 
                                                                                          !! @tex ($\mu mol m^{-2} s^{-1}$) @endtex
 
  
    !
    !! 0.3 Modified variables
    !
    REAL(r_std), DIMENSION(npts,nvm,nleafages), INTENT(inout)  :: leaf_age                !! Leaf age (days)
    REAL(r_std), DIMENSION(npts,nvm,nleafages), INTENT(inout)  :: leaf_frac               !! fraction of leaves in leaf age 
                                                                                          !! classes 
                                                                                          !! (unitless)

    !
    !! 0.4 Local variables
    !
    REAL(r_std), DIMENSION(npts)                               :: leaf_efficiency         !! leaf efficiency (vcmax/vcmax25)
                                                                                          !! (unitless)
    REAL(r_std), DIMENSION(npts,nvm,nleafages)                 :: d_leaf_frac             !! turnover between age classes
                                                                                          !! (unitless)
    REAL(r_std), DIMENSION(npts,nleafages)                     :: leaf_age_new            !! new leaf age (days)
    REAL(r_std), DIMENSION(npts)                               :: sumfrac                 !! sum of leaf age fractions, 
                                                                                          !! for normalization
                                                                                          !! (unitless)
    REAL(r_std), DIMENSION(npts)                               :: rel_age                 !! relative leaf age (age/critical age)
                                                                                          !! (unitless)
    INTEGER(i_std)                                             :: i,j,m                   !! indices (unitless)
    REAL(r_std), DIMENSION(npts,nvm)                           :: age_decline             !! Age-related decline in photosynthesis efficiency 
                                                                                          !! (as a proportion to vcmax and vjmax : 1 while age<60
                                                                                          !! (cf. Gower 1996) 
                                                                                          !!  and decline_max when age>140 (cf Gower 1996))
    REAL(r_std), SAVE                                          :: ss_decl_max             !! Temporary variable for sensitivity study 
!$OMP THREADPRIVATE(ss_decl_max)

!_ ================================================================================================================================

    IF (bavard.GE.3) WRITE(numout,*) 'Entering vmax'

    !
    !! 1 Initialization
    !

    !
    !! 1.1 first call: info about flags and parameters.
    !

    IF ( firstcall ) THEN

! We do not think we need the FM stuff here anymore.  In theory,
! the only difference between managed trees and unmanaged trees
! should be the thinning operations.
!!$       IF ( control%forest_management ) THEN
!!$          
!!$          ss_decl_max = 1.
!!$          !Config Key   = 
!!$          !Config Desc  = 
!!$          !Config If    = 
!!$          !Config Def   = 
!!$          !Config Help  = 
!!$          !Config Units = 
!!$          CALL getin_p ('ss_decl_max',ss_decl_max)
!!$         
!!$       END IF ! control%forest_management

       WRITE(numout,*) 'vmax:'

       WRITE(numout,*) '   > offset (minimum vcmax/vmax_opt):' , vmax_offset
       WRITE(numout,*) '   > relative leaf age at which vmax reaches vcmax_opt:', leafage_firstmax 
       WRITE(numout,*) '   > relative leaf age at which vmax falls below vcmax_opt:', leafage_lastmax
       WRITE(numout,*) '   > relative leaf age at which vmax reaches its minimum:', leafage_old

       firstcall = .FALSE.

    ENDIF

    !
    !! 1.2 initialize output
    !
    age_decline(:,:) = zero
    vcmax(:,:) = zero

    !
    !! 2 leaf age: general increase and turnover between age classes.
    !

    !
    !! 2.1 increase leaf age
    !
!
!! The age of the leaves in each leaf-age-class increases by 1 time step.
    DO m = 1, nleafages	! Loop over # leaf age classes 
       DO j = 2,nvm	! Loop over # PFTs
          WHERE ( leaf_frac(:,j,m) .GT. min_stomate )

             leaf_age(:,j,m) = leaf_age(:,j,m) + dt
             
          ENDWHERE
       ENDDO	! Loop over # PFTs 

    ENDDO	! Loop over # leaf age classes

    !
    !! 2.2 turnover between leaf age classes
    !     d_leaf_frac(:,:,m) = what leaves m-1 and goes into m
    !

    DO j = 2,nvm	! Loop over # PFTs

       !! 2.2.1 fluxes

       !! nothing goes into first age class
       d_leaf_frac(:,j,1) = zero

       !! for others age classes (what goes from m-1 to m)
       DO m = 2, nleafages 
!! leaf_timecst is defined in pft_parameters.f90 as the quotient of the critical leaf age per the number of age classes.
!! The critical leaf age is a PFT-dependent constant defined in pft_parameters.f90, that represents the leaf life span.
!! This time constant (leaf_timecst) determines the turnover between the nleafages different leaf age classes
!! (see section [118] in Krinner et al. (2005)).
          d_leaf_frac(:,j,m) = leaf_frac(:,j,m-1) * dt/leaf_timecst(j)

       ENDDO

       !! 2.2.2 new leaf age in class
       !!       new age = ( old age * (old fraction - fractional loss) + fractional increase * age of the source class ) / new fraction
       !!       The leaf age of the youngest class (m=1) is updated into stomate_alloc          
       leaf_age_new(:,:) = zero

       DO m = 2, nleafages-1       ! Loop over age classes
	!! For all age classes except first and last 
          WHERE ( d_leaf_frac(:,j,m) .GT. min_stomate )

             leaf_age_new(:,m) = ( ( (leaf_frac(:,j,m)- d_leaf_frac(:,j,m+1)) * leaf_age(:,j,m) )  + &
                  ( d_leaf_frac(:,j,m) * leaf_age(:,j,m-1) ) ) / &
                  ( leaf_frac(:,j,m) + d_leaf_frac(:,j,m)- d_leaf_frac(:,j,m+1) )

          ENDWHERE

       ENDDO       ! Loop over age classes

	!! For last age class, there is no leaf fraction leaving the class. 

       WHERE ( d_leaf_frac(:,j,nleafages) .GT. min_stomate )

          leaf_age_new(:,nleafages) = ( ( leaf_frac(:,j,nleafages) * leaf_age(:,j,nleafages) )  + &
               ( d_leaf_frac(:,j,nleafages) * leaf_age(:,j,nleafages-1) ) ) / &
               ( leaf_frac(:,j,nleafages) + d_leaf_frac(:,j,nleafages) )

       ENDWHERE

       DO m = 2, nleafages       ! Loop over age classes

          WHERE ( d_leaf_frac(:,j,m) .GT. min_stomate )

             leaf_age(:,j,m) = leaf_age_new(:,m)

          ENDWHERE

       ENDDO       ! Loop over age classes

       !! 2.2.3 calculate new fraction

       DO m = 2, nleafages       ! Loop over age classes

          ! where the change comes from
          leaf_frac(:,j,m-1) = leaf_frac(:,j,m-1) - d_leaf_frac(:,j,m)

          ! where it goes to
          leaf_frac(:,j,m) = leaf_frac(:,j,m) + d_leaf_frac(:,j,m)

       ENDDO       ! Loop over age classes

       !! 2.2.4 renormalize fractions in order to prevent accumulation 
       !       of numerical errors

       ! correct small negative values

       DO m = 1, nleafages
          leaf_frac(:,j,m) = MAX( zero, leaf_frac(:,j,m) )
       ENDDO

       ! total of fractions, should be very close to one where there is leaf mass

       sumfrac(:) = zero

       DO m = 1, nleafages       ! Loop over age classes

          sumfrac(:) = sumfrac(:) + leaf_frac(:,j,m)

       ENDDO       ! Loop over age classes

       ! normalize

       DO m = 1, nleafages       ! Loop over age classes

          WHERE ( sumfrac(:) .GT. min_stomate )

             leaf_frac(:,j,m) = leaf_frac(:,j,m) / sumfrac(:) 

          ELSEWHERE

             leaf_frac(:,j,m) = zero

          ENDWHERE

       ENDDO       ! Loop over age classes

    ENDDO         ! Loop over PFTs

    !
    !! 3 calculate vmax as a function of the age
    !

    DO j = 2,nvm

       vcmax(:,j) = zero
 

       ! sum up over the different age classes

       DO m = 1, nleafages       ! Loop over age classes

          !
          !! 3.1 efficiency in each of the age classes
          !!     it varies from vmax_offset to 1 
          !!     linearly increases from vmax_offset to 1 for 0 < rel_age < leafage_firstmax
          !!     is 1 when leafage_firstmax < rel_age < leafage_lastmax
          !!     linearly decreases from 1 to vmax_offset for leafage_lastmax < rel_age < leafage_firstmax
          !!     vmax_offset for rel_age >= leafage_old
          !!     (Ishida et al., 1999)

          rel_age(:) = leaf_age(:,j,m) / tau_leaf(j)

          leaf_efficiency(:) = MAX( vmax_offset, MIN( un, &
               vmax_offset + (un - vmax_offset) * rel_age(:) / leafage_firstmax, &
               un - (un - vmax_offset) * ( rel_age(:) - leafage_lastmax ) / &
               ( leafage_old - leafage_lastmax ) ) )


          !! 3.2 add to mean vmax
          vcmax(:,j) = vcmax(:,j) + Vcmax25(j) * leaf_efficiency(:) * leaf_frac(:,j,m)
          


       ENDDO     ! loop over age classes

       !
       !! 4 Modify vmax as a function of the age of the stand and management
       !
! We do not think we need the FM stuff here anymore.  In theory,
! the only difference between managed trees and unmanaged trees
! should be the thinning operations.
!!$       IF ( control%forest_management ) THEN
!!$
!!$          IF (is_tree(j)) THEN
!!$
!!$             WHERE ( forest_managed(:,j) == 0 )
!!$                age_decline(:,j) = 1.        !-ss_decl_max*decl_factor(j)*100.
!!$             END WHERE
!!$
!!$             DO i = 1,npts
!!$                IF (forest_managed(i,j) > 0) THEN
!!$
!!$                   ! If TCR, values from Calfapietra 2005
!!$                   IF (forest_managed(i,j) == 4) THEN
!!$                      vcmax(i,j) = vcmax(i,j)*79.1/vcmax_opt(j)
!!$                      vjmax(i,j) = vjmax(i,j)*152.4/vjmax_opt(j)
!!$                   END IF
!!$                   ! Final age-related decline in photosynthesis efficiency
!!$                   age_decline(i,j) = 1.-ss_decl_max*decl_factor(j)*REAL(MIN(MAX(age_stand(i,j)-50,0),100)) !**0.5
!!$
!!$                END IF
!!$             END DO
!!$
!!$             vcmax(:,j) = age_decline(:,j)**4*opt_factor(j)*vcmax(:,j)
!!$             vjmax(:,j) = age_decline(:,j)**4*opt_factor(j)*vjmax(:,j)
!!$
!!$          ENDIF
!!$
!!$       END IF ! (control%forest_management)

    ENDDO       ! loop over PFTs

    IF (bavard.GE.4) WRITE(numout,*) 'Leaving vmax'

  END SUBROUTINE vmax_diagnostic

END MODULE stomate_vmax
